@@ -37,14 +37,28 @@ char *path_ypdb = _PATH_YP;
 
 int debug_flag = 1;
 
+
+static void Gperror(char *str)
+{
+    fprintf(stderr, "%s: GDBM Error Code #%d\n", gdbm_errno);
+}
+
+
 void *ypproc_null_2_svc(void *dummy, struct svc_req *rqstp)
 {
     static int foo;
 
+    
+    if (debug_flag)
+	fprintf(stderr, "ypproc_null()\n");
+
     return (void *) &foo;
 }
 
-int is_valid_domain(char *name)
+/*
+** Return 1 if the name is a valid domain name served by us, else 0.
+*/
+static int is_valid_domain(char *name)
 {
     char buf[1025];
     struct stat sbuf;
@@ -57,16 +71,14 @@ int is_valid_domain(char *name)
     return 1;
 }
 
+
 bool_t *ypproc_domain_2_svc(domainname *name, struct svc_req *rqstp)
 {
     static bool_t result;
 
 
     if (debug_flag)
-    {
-	fprintf(stderr, "ypproc_domain(\"%s\")\n",
-		*name);
-    }
+	fprintf(stderr, "ypproc_domain(\"%s\")\n", *name);
 
     if (is_valid_domain(*name))
 	result = TRUE;
@@ -80,20 +92,20 @@ bool_t *ypproc_domain_2_svc(domainname *name, struct svc_req *rqstp)
     return &result;
 }
 
+
 bool_t *ypproc_domain_nonack_2_svc(domainname *name, struct svc_req *rqstp)
 {
     static bool_t result;
 
     if (debug_flag)
-    {
-	fprintf(stderr, "ypproc_domain_nonack(\"%s\")\n",
-		*name);
-    }
+	fprintf(stderr, "ypproc_domain_nonack(\"%s\")\n", *name);
     
     if (!is_valid_domain(*name))
     {
 	if (debug_flag)
 	    fprintf(stderr, "\t-> Not a valid domain.\n");
+
+	/* Bail out and don't return any RPC value */
 	return NULL;
     }
     
@@ -110,16 +122,21 @@ static GDBM_FILE open_database(char *domain, char *map)
     GDBM_FILE dbp;
     char buf[1025];
 
+    
     sprintf(buf, "%s/%s/%s", path_ypdb, domain, map);
     
-    dbp = gdbm_open(buf, 60, O_RDONLY, 0700, NULL);
+    dbp = gdbm_open(buf, 0, O_RDONLY, 0, NULL);
 
     if (debug_flag && dbp == NULL)
-	fprintf(stderr, "*** Failed domain=%s, map=%s\n", domain, map);
+	Gperror("gdbm_open()");
     
     return dbp;
 }
 
+
+/*
+** Get the DateTimeModified value for a certain map database
+*/
 static unsigned long get_dtm(char *domain, char *map)
 {
     struct stat sbuf;
@@ -140,16 +157,25 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key, struct svc_req *rqstp)
     if (debug_flag)
     {
 	fprintf(stderr, "ypproc_match():\n");
-	fprintf(stderr, "\tdomainname = \"%s\"\n", key->domain);
-	fprintf(stderr, "\tmapname = \"%s\"\n", key->map);
-	fprintf(stderr, "\tkeydat = \"%.*s\"\n", key->key.keydat_len,
+	fprintf(stderr, "\t\tdomainname = \"%s\"\n",
+		key->domain);
+	fprintf(stderr, "\t\tmapname = \"%s\"\n",
+		key->map);
+	fprintf(stderr, "\t\tkeydat = \"%.*s\"\n",
+		key->key.keydat_len,
 		key->key.keydat_val);
     }
 
     result.val.valdat_len = 0;
-    result.val.valdat_val = NULL;
+    if (result.val.valdat_val)
+    {
+	free(result.val.valdat_val);
+	result.val.valdat_val = NULL;
+    }
     
-    if (!is_valid_domain(key->domain))
+    if (key->domain[0] == '\0' || key->map[0] == '\0')
+	result.stat = YP_BADARGS;
+    else if (!is_valid_domain(key->domain))
 	result.stat = YP_NODOM;
     else
     {
@@ -165,15 +191,19 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key, struct svc_req *rqstp)
 
 	    rdat = gdbm_fetch(dbp, qdat);
 	    if (rdat.dptr == NULL)
+	    {
+		if (debug_flag)
+		    Gperror("gdbm_fetch()");
 		result.stat = YP_NOKEY;
+	    }
 	    else
 	    {
 		result.stat = YP_TRUE;
 		result.val.valdat_len = rdat.dsize;
 		result.val.valdat_val = rdat.dptr;
 	    }
+	    gdbm_close(dbp);
 	}
-	gdbm_close(dbp);
     }
 	
     return &result;
@@ -194,11 +224,22 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_key *key, struct svc_req *rqstp)
     }
 
     result.key.keydat_len = 0;
-    result.key.keydat_val = NULL;
-    result.val.valdat_len = 0;
-    result.val.valdat_val = NULL;
+    if (result.key.keydat_val)
+    {
+	free(result.key.keydat_val);
+	result.key.keydat_val = NULL;
+    }
     
-    if (!is_valid_domain(key->domain))
+    result.val.valdat_len = 0;
+    if (result.val.valdat_val)
+    {
+	free(result.val.valdat_val);
+	result.val.valdat_val = NULL;
+    }
+
+    if (key->map[0] == '\0' || key->domain[0] == '\0')
+	result.stat = YP_BADARGS;
+    else if (!is_valid_domain(key->domain))
 	result.stat = YP_NODOM;
     else
     {
@@ -231,6 +272,7 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_key *key, struct svc_req *rqstp)
     return &result;
 }
 
+
 ypresp_key_val *ypproc_next_2_svc(ypreq_key *key, struct svc_req *rqstp)
 {
     static ypresp_key_val result;
@@ -246,16 +288,28 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key, struct svc_req *rqstp)
     }
 
     result.key.keydat_len = 0;
-    result.key.keydat_val = NULL;
-    result.val.valdat_len = 0;
-    result.val.valdat_val = NULL;
+    if (result.key.keydat_val)
+    {
+	free(result.key.keydat_val);
+	result.key.keydat_val = NULL;
+    }
     
-    if (!is_valid_domain(key->domain))
+    result.val.valdat_len = 0;
+    if (result.val.valdat_val)
+    {
+	free(result.val.valdat_val);
+	result.val.valdat_val = NULL;
+    }
+    
+    if (key->map[0] == '\0' || key->domain[0] == '\0')
+	result.stat = YP_BADARGS;
+    else if (!is_valid_domain(key->domain))
 	result.stat = YP_NODOM;
     else
     {
 	datum dkey, dval;
-	
+
+
 	GDBM_FILE dbp = open_database(key->domain, key->map);
 	if (dbp == NULL)
 	    result.stat = YP_NOMAP;
@@ -266,22 +320,36 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key, struct svc_req *rqstp)
 	    
 	    dkey = gdbm_nextkey(dbp, dkey);
 	    if (dkey.dptr == NULL)
+	    {
+		if (debug_flag)
+		    Gperror("gdbm_nextkey()");
+		
 		result.stat = YP_NOMORE;
+	    }
 	    else
 	    {
 		dval = gdbm_fetch(dbp, dkey);
-		
-		result.stat = YP_TRUE;
-		
-		result.key.keydat_len = dkey.dsize;
-		result.key.keydat_val = dkey.dptr;
-		
-		result.val.valdat_len = dval.dsize;
-		result.val.valdat_val = dval.dptr;
+		if (dval.dptr == NULL)
+		{
+		    if (debug_flag)
+			Gperror("gdbm_fetch()");
+		    
+		    result.stat = YP_BADDB;
+		}
+		else
+		{
+		    result.stat = YP_TRUE;
+		    
+		    result.key.keydat_len = dkey.dsize;
+		    result.key.keydat_val = dkey.dptr;
+		    
+		    result.val.valdat_len = dval.dsize;
+		    result.val.valdat_val = dval.dptr;
+		}
 	    }
+	    
+	    gdbm_close(dbp);
 	}
-
-	gdbm_close(dbp);
     }
 
     return &result;
@@ -295,6 +363,7 @@ static void print_ypmap_parms(struct ypmap_parms *pp)
     fprintf(stderr, "\t\tpeer     = \"%s\"\n", pp->peer);
 }
 
+
 ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr, struct svc_req *rqstp)
 {
     static ypresp_xfr result;
@@ -303,9 +372,10 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr, struct svc_req *rqstp)
     {
 	fprintf(stderr, "ypproc_xfr_2_svc():\n\tmap_parms:\n");
 	print_ypmap_parms(&xfr->map_parms);
-	fprintf(stderr, "\ttransid = %u\n", xfr->transid);
-	fprintf(stderr, "\tprog = %u\n", xfr->prog);
-	fprintf(stderr, "\tport = %u\n", xfr->port);
+	fprintf(stderr, "\t\ttransid = %u\n", xfr->transid);
+	fprintf(stderr, "\t\tprog = %u\n", xfr->prog);
+	fprintf(stderr, "\t\tport = %u\n", xfr->port);
+	fprintf(stderr, "\t-> Request denied.\n");
     }
 
     result.transid = xfr->transid;
@@ -314,6 +384,7 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr, struct svc_req *rqstp)
     return &result;
 }
 
+
 void *ypproc_clear_2_svc(void *dummy, struct svc_req *rqstp)
 {
     static int foo;
@@ -326,149 +397,51 @@ void *ypproc_clear_2_svc(void *dummy, struct svc_req *rqstp)
 
 
 
-/*
-** Hmm... I wonder how on Earth this one is supposed to work. Perhaps
-** like this?
-*/
 ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
 {
-    static ypresp_all *result = NULL;
-    static int reslen = 0;
-    GDBM_FILE dbp;
-    int i;
+    static ypresp_all result;
     
     
     if (debug_flag)
     {
 	fprintf(stderr, "ypproc_all_2_svc():\n");
-	fprintf(stderr, "\tdomain = \"%s\"\n", nokey->domain);
-	fprintf(stderr, "\tmap = \"%s\"\n", nokey->map);
-    }
-
-    if (result == NULL)
-    {
-	result = malloc(sizeof(*result));
-	if (result == NULL)
-	{
-	    /* Fatal error. Should babble to syslog et al... */
-	    return NULL;
-	}
-	reslen = 0;
-    }
-
-    i = 0;
-	
-    
-    if ((dbp = open_database(nokey->domain, nokey->map)) == NULL)
-	result->ypresp_all_u.val.stat = YP_NODOM;
-    else
-#if 1
-    {
-	result[i].more = 0;
-	result[i].ypresp_all_u.val.stat = YP_YPERR;
-	result[i].ypresp_all_u.val.key.keydat_len = 0;
-	result[i].ypresp_all_u.val.key.keydat_val = NULL;
-	result[i].ypresp_all_u.val.val.valdat_len = 0;
-	result[i].ypresp_all_u.val.val.valdat_val = NULL;
-    }
-#else
-    {
-	datum key, val;
-
-	for (key = gdbm_firstkey(dbp); key.dptr; key = gdbm_nextkey(dbp, key))
-	{
-	    if (i >= reslen)
-	    {
-		int j;
-
-		reslen += 32;
-		if (debug_flag)
-		    fprintf(stderr, "\t(Reallocating buffer to %d slots)\n",
-			    reslen);
-		
-		result = realloc(result, sizeof(*result)*(reslen+1));
-		if (result == NULL)
-		{
-		    /* Fatal error. Should babble to syslog et al... */
-		    reslen = 0;
-		    gdbm_close(dbp);
-		    return NULL;
-		}
-
-		for (j = i; j <= reslen; j++)
-		{
-		    result[j].more = 0;
-		    result[j].ypresp_all_u.val.key.keydat_len = 0;
-		    result[j].ypresp_all_u.val.key.keydat_val = NULL;
-		    result[j].ypresp_all_u.val.val.valdat_len = 0;
-		    result[j].ypresp_all_u.val.val.valdat_val = NULL;
-		}
-	    }
-	    
-	    if (result[i].ypresp_all_u.val.key.keydat_val)
-	    {
-		free(result[i].ypresp_all_u.val.key.keydat_val);
-		result[i].ypresp_all_u.val.key.keydat_val = NULL;
-	    }
-	    
-	    if (result[i].ypresp_all_u.val.val.valdat_val)
-	    {
-		free(result[i].ypresp_all_u.val.val.valdat_val);
-		result[i].ypresp_all_u.val.val.valdat_val = NULL;
-	    }
-
-	    val = gdbm_fetch(dbp, key);
-
-/* XXXX check val.dptr... */	    
-	    
-	    if (debug_flag)
-		fprintf(stderr,
-			"\t(Inserting at slot %d: key = \"%.*s\", value = \"%.*s\")\n",
-			i, key.dsize, key.dptr, val.dsize, val.dptr);
-			
-	    result[i].more = 1;
-	    result[i].ypresp_all_u.val.stat = YP_TRUE;
-	    result[i].ypresp_all_u.val.key.keydat_len = key.dsize;
-	    result[i].ypresp_all_u.val.key.keydat_val = strdup(key.dptr);
-	    result[i].ypresp_all_u.val.val.valdat_len = val.dsize;
-	    result[i].ypresp_all_u.val.val.valdat_val = strdup(val.dptr);
-
-	    i++;
-	}
-	     
-	gdbm_close(dbp);
+	fprintf(stderr, "\t\tdomain = \"%s\"\n", nokey->domain);
+	fprintf(stderr, "\t\tmap = \"%s\"\n", nokey->map);
+	fprintf(stderr, "\t-> Request denied (not yet implemented).\n");
     }
     
-    if (debug_flag)
-	fprintf(stderr, "\t-> Returning %d values\n", i);
+    result.more = 0;
+    result.ypresp_all_u.val.stat = YP_YPERR;
+    result.ypresp_all_u.val.key.keydat_len = 0;
+    result.ypresp_all_u.val.key.keydat_val = NULL;
+    result.ypresp_all_u.val.val.valdat_len = 0;
+    result.ypresp_all_u.val.val.valdat_val = NULL;
     
-    result[i].more = 0;
-
-    result[i].ypresp_all_u.val.stat = YP_NOMORE;
-    result[i].ypresp_all_u.val.key.keydat_len = 0;
-    result[i].ypresp_all_u.val.key.keydat_val = NULL;
-    result[i].ypresp_all_u.val.val.valdat_len = 0;
-    result[i].ypresp_all_u.val.val.valdat_val = NULL;
-
-#endif
-
-    return result;
+    return &result;
 }
 
+
 ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
 {
     static ypresp_master result;
 
+    
     if (debug_flag)
     {
 	fprintf(stderr, "ypproc_master_2_svc():\n");
-	fprintf(stderr, "\tdomain = \"%s\"\n", nokey->domain);
-	fprintf(stderr, "\tmap = \"%s\"\n", nokey->map);
+	fprintf(stderr, "\t\tdomain = \"%s\"\n", nokey->domain);
+	fprintf(stderr, "\t\tmap = \"%s\"\n", nokey->map);
     }
 
-    result.peer = "";
+    if (result.peer)
+    {
+	free(result.peer);
+	result.peer = NULL;
+    }
 
-    if (!is_valid_domain(nokey->domain))
+    if (nokey->domain[0] == '\0')
+	result.stat = YP_BADARGS;
+    else if (!is_valid_domain(nokey->domain))
 	result.stat = YP_NODOM;
     else
     {
@@ -489,7 +462,7 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
 		static char hostbuf[1025];
 
 		gethostname(hostbuf, sizeof(hostbuf)-1);
-		result.peer = hostbuf;
+		result.peer = strdup(hostbuf);
 	    }
 	    else
 		result.peer = val.dptr;
@@ -499,9 +472,16 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
 	}
     }
 
+    if (result.peer == NULL)
+	result.peer = strdup("");
+
+    if (debug_flag)
+	fprintf(stderr, "\t-> Peer = \"%s\"\n", result.peer);
+    
     return &result;
 }
 
+
 ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
 {
     static ypresp_order result;
@@ -509,13 +489,15 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
     if (debug_flag)
     {
 	fprintf(stderr, "ypproc_order_2_svc():\n");
-	fprintf(stderr, "\tdomain = \"%s\"\n", nokey->domain);
-	fprintf(stderr, "\tmap = \"%s\"\n", nokey->map);
+	fprintf(stderr, "\t\tdomain = \"%s\"\n", nokey->domain);
+	fprintf(stderr, "\t\tmap = \"%s\"\n", nokey->map);
     }
 
     result.ordernum = 0;
 
-    if (!is_valid_domain(nokey->domain))
+    if (nokey->domain[0] == '\0')
+	result.stat = YP_BADARGS;
+    else if (!is_valid_domain(nokey->domain))
 	result.stat = YP_NODOM;
     else
     {
@@ -536,13 +518,19 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey, struct svc_req *rqstp)
 		result.ordernum = get_dtm(nokey->domain, nokey->map);
 	    }
 	    else
+	    {
 		result.ordernum = atoi(val.dptr);
+		free(val.dptr);
+	    }
 
 	    result.stat = YP_TRUE;
 	    gdbm_close(dbp);
 	}
     }
 
+    if (debug_flag)
+	fprintf(stderr, "-> Order # %d\n", result.ordernum);
+    
     return &result;
 }
 
@@ -589,7 +577,7 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name, struct svc_req *rqstp)
     if (debug_flag)
     {
 	fprintf(stderr, "ypproc_maplist_2_svc():\n");
-	fprintf(stderr, "\tdomain = \"%s\"\n", *name);
+	fprintf(stderr, "\t\tdomain = \"%s\"\n", *name);
     }
 
     if (result.maps)
@@ -597,7 +585,9 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name, struct svc_req *rqstp)
     
     result.maps = NULL;
 
-    if (!is_valid_domain(*name))
+    if ((*name)[0] == '\0')
+	result.stat = YP_BADARGS;
+    else if (!is_valid_domain(*name))
 	result.stat = YP_NODOM;
     else
     {
@@ -605,13 +595,22 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name, struct svc_req *rqstp)
 
 	dp = opendir(path_ypdb);
 	if (dp == NULL)
+	{
+	    if (debug_flag)
+		perror("opendir");
+	    
 	    result.stat = YP_BADDB;
+	}
 	else
 	{
 	    struct dirent *dep;
 	    
 	    while (dep = readdir(dp))
-		add_maplist(&result.maps, dep->d_name);
+		if (add_maplist(&result.maps, dep->d_name) < 0)
+		{
+		    result.stat = YP_YPERR;
+		    break;
+		}
 	}
 
 	closedir(dp);
